/* $Header: minopt.h,v 3.2 98/08/13 17:18:21 carl Exp $ */

/************************************************************************
 * Copyright (c) 1998 The Trustees of Princeton University.
 * All rights reserved.
 * 
 * PRINCETON UNIVERSITY MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF
 * THIS SOFTWARE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS
 * OR IMPLIED WARRANTY.  PRINCETON UNIVERSITY SHALL NOT BE LIABLE FOR ANY
 * DAMAGES SUFFERED BY THE USERS OF THIS SOFTWARE.
 ************************************************************************/


#ifndef _MINOPT_INCLUDED
#define _MINOPT_INCLUDED

/* Contains data structure for the problem */

#include "config.h"
#include "termdata.h"
#include "option.h"
#include "formout.h"
#include "cmd.h"

/* PROBLEM STRUCTURES */

typedef struct row {            /* Row (constraint) information */
  unsigned long int rtype;      /* type of the row */
  char *name;                   /* name of the row */

  int nterms;                   /* number of linear + bilinear terms */
  TERMTYPE terms;               /* linear/bilinear terms */

  int nut;                      /* number of univariate terms */
  int *utind;                   /* Array of univariate term indices */
  int *utvar[4];                /* Array of variable indices for univ. terms */
  int *addvar;                  /* Array of additional variables arising 
				   from the bilinear terms */
  double coeff;                 /* coefficient of bilinear term (for aBB) */

  int nimpd;                    /* number of implicit discontinuities */
  int nexpd;                    /* number of explicit discontinuities */

  cmdlist *c;                   /* constraint function */
  cmdlist **dcdx;               /* derivative of c wrt x */
  cmdlist **dcdy;               /* derivative of c wrt y */
  cmdlist **dcdz;               /* derivative of c wrt z */
  cmdlist **dcdu;               /* derivative of c wrt u */
  
  int *xcol;                    /* columns in which the derivatives wrt x are
				   nonzero (for sparse structure)
				   (variable numbers start at 0) */
  int *ycol;                    /* columns in which the derivatives wrt y are
				   nonzero (for sparse structure)
				   (variable numbers start at 0) */
  int *zcol;                    /* columns in which the derivatives wrt z are
				   nonzero (for sparse structure)
				   (variable numbers start at 0) */
  int *ucol;                    /* columns in which the derivatives wrt u are
				   nonzero (for sparse structure)
				   (variable numbers start at 0) */
  
  int ndcdx;                    /* number of x derivatives for both linear
				   and nonlinear constraints (dimension for
				   dcdx and xcol in nonlinear case) */
  int ndcdy;                    /* number of y derivatives for both linear
				   and nonlinear constraints (dimension for
				   dcdy and ycol in nonlinear case) */
  int ndcdz;                    /* number of z derivatives for both linear
				   and nonlinear constraints (dimension for
				   dcdz and zcol in nonlinear case) */
  int ndcdu;                    /* number of u derivatives for both linear
				   and nonlinear constraints (dimension for
				   dcdz and zcol in nonlinear case) */
  
  int sense;                    /* sense of the constraints */
  double rhs;                   /* Right hand side of constraints */
  int point;                    /* end of which interval point constraint is in effect */
  
} *ROWTYPE, **ROWPTR;           /* ROWTYPE points to data for a row */
                                /* ROWPTR is set of pointers of rows */

typedef struct problem {        /* Main MINOPT problem data structure */
  char *name;                   /* Problem name */
  char *basename;               /* input/output file base name */
  char *infilename;             /* name of the input file */
  char *outfilename;            /* name of the output file */
  FILE *fp;                     /* Output file--NULL for no output */

  int nx;                       /* Number of X variables */
  int ny;                       /* Number of Y variables */
  int nz;                       /* Number of Z (dynamic) variables */
  int nu;                       /* Number of U (control) variables */

  char **xnames;                /* names of x variables */
  char **ynames;                /* names of y variables */
  char **znames;                /* names of z variables */
  char **unames;                /* names of u variables */

  int nlinrows;                 /* No. of linear/bilinear rows */
  int nnlnrows;                 /* No. of nonlinear rows */
  int ndlinrows;                /* No. of linear dual linear rows */
  int ndnlnrows;                /* No. of nonlinear dual nonlinear rows */
  int nylinrows;                /* No. of y only linear rows */
  int nynlnrows;                /* No. of y only nonlinear rows */
  int ndynrows;                 /* No. of dynamic rows */
  int npointrows;               /* No. of point constraints */
  int npathrows;                /* No. of path constraints */
  int tnrows;                   /* Total no. of constraints */

  double *xlbd;                 /* Lower bounds on X */
  double *xubd;                 /* Upper bounds on X */
  double *ylbd;                 /* Lower bounds on Y */
  double *yubd;                 /* Upper bounds on Y */
  double *zlbd;                 /* Lower bounds on Z */
  double *zubd;                 /* Upper bounds on Z */
  double *ulbd;                 /* Lower bounds on U */
  double *uubd;                 /* Upper bounds on U */

  char *xtype;                  /* Type of X variables */
  char *ytype;                  /* Type of Y variables */

  double *xstp;                 /* starting point for the x-variables */
  double *ystp;                 /* starting point for the y-variables */
  double *zinc;                 /* Initial condition for  z-variables */
  double *ustp;                 /* starting point for the u-variables */

  int *xpri;                    /* priority on the x-variables for MILP */
  int *ypri;                    /* priority on the y-variables for MILP */

  ROWTYPE rows;                 /* Objective row (Data for constraints) */

  ROWPTR linrows;               /* Linear/bilinear rows */
  ROWPTR nlnrows;               /* Nonlinear rows */
  ROWPTR dlinrows;              /* Rows going to dual problem as is */
  ROWPTR dnlnrows;              /* Rows going to dual problem as is */
  ROWPTR ylinrows;              /* Rows involving y only */
  ROWPTR ynlnrows;              /* Rows involving y only */
  ROWPTR dynrows;               /* Dynamic rows */
  ROWPTR pointrows;             /* point constraints */
  ROWPTR pathrows;              /* path constraints */

  int *ncpt;                    /* Number of collocation points for each control or
				   Order of the polynomial approximating the control */
  int ntime;                    /* number of time instances (intervals+1) */
  double *time;                 /* time instances for integration */

  int *ispec;                   /* indicies of variables used to determine initial condition */

  int nincopt;                  /* number of initial conditions to be optimized */
  int *incstate;                /* states whose initial conditions are to be optimized */
  int *incparam;                /* parameters for initial conditions to be optimized */

  char *part_set;               /* Constraint partition for GCD */

  OPTIONTYPE minopt_options;    /* Options for MINOPT */
  OPTIONTYPE minos_options;     /* Options for MINOS */
  OPTIONTYPE snopt_options;     /* Options for SNOPT */
  OPTIONTYPE npsol_options;     /* Options for NPSOL */
  OPTIONTYPE cplex_options;     /* Options for CPLEX */
  OPTIONTYPE lpsolve_options;   /* Optinos for LPSOLVE */
  OPTIONTYPE dasolv_options;    /* Options for DASOLV */
  OPTIONTYPE daspk_options;     /* Options for DASPK */
  OPTIONTYPE daeopt_options;    /* Options for DAEOPT */

  FORMOUTTYPE formatted_output; /* formatted output to be printed after solution */

  int numoutrows;               /* number of rows for output */
  ROWPTR outrows;               /* output that is stored as a row */


  /* The parts of the problem structure listed below can be determined from the
     other information in the problem structure.  They will be set up by the
     MINOPT presolve.  If MINOPT is being called as a subroutine and the
     following information is set up by the user, the presolve can be bypassed
     by setting nopresolve to 1.  */

  int presolve;                 /* 1 to have MINOPT do the presolve, 0 otherwise */

  unsigned long int ptype;      /* Type of problem */
  int nzx;                      /* Total non-zeros in x-jacobian */
  int nzy;                      /* Total non-zeros in y-jacobian */
  int nzz;                      /* Total non-zeros in z-jacobian */
  int nzu;                      /* Total non-zeros in u-jacobian */
  int nxsen;                    /* Number of X variables in the DAE system
				   Thus requiring sensitivity analysis */
  int *xsen;                    /* indices of the x variables in the DAE system
				   mapping for the indices of xsen to x */
  int *xsencol;                 /* mapping for the indices of x to xsen */
  int *diffrow;                 /* 1 if the dynamic row is differential, 0 otherwise */
  int *diffvar;                 /* 1 if the dynamic variable is differential, 0 otherwise */
  int numdiff;                  /* the number of differential terms (and usually rows) */
  int need_adjoint;             /* whether or not it is necessary to solve an adjoint problem */


} *PROBLEMTYPE;

#endif /* _MINOPT_INCLUDED */ 
